"""
 @file
 @brief Unit tests for tree-view keyboard search
 @author OpenShot Studios, LLC

 @section LICENSE

 Copyright (c) 2008-2026 OpenShot Studios, LLC
 (http://www.openshotstudios.com). This file is part of
 OpenShot Video Editor (http://www.openshot.org), an open-source project
 dedicated to delivering high quality video editing and animation solutions
 to the world.

 OpenShot Video Editor is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 OpenShot Video Editor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
 """

import unittest

from qt_api import QApplication, QAbstractItemView, QStandardItem, QStandardItemModel
from qt_api import QTreeView

from windows.views.name_search_treeview import NameColumnKeyboardSearchMixin


class SearchTreeView(NameColumnKeyboardSearchMixin, QTreeView):
    pass


class NameColumnKeyboardSearchTests(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.app = QApplication.instance() or QApplication([])

    def test_keyboard_search_uses_name_instead_of_blank_thumbnail_column(self):
        model = QStandardItemModel()
        for name in ("Alpha", "Bravo", "Charlie"):
            model.appendRow([QStandardItem(""), QStandardItem(name)])

        view = SearchTreeView()
        view.setModel(model)
        view.setSelectionBehavior(QAbstractItemView.SelectRows)
        view.setCurrentIndex(model.index(0, 0))

        view.keyboardSearch("c")

        current = view.currentIndex()
        self.assertEqual(current.row(), 2)
        self.assertEqual(current.column(), 1)
        self.assertEqual(current.data(), "Charlie")


if __name__ == "__main__":
    unittest.main()



"""
 @file
 @brief Unit tests for Color Grade preset helpers
"""

import os
import sys
import unittest

PATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
if PATH not in sys.path:
    sys.path.append(PATH)

from classes.color_presets import (  # noqa: E402
    COLOR_PRESET_AUTO_CONTRAST,
    COLOR_PRESET_BOOST_COLOR,
    COLOR_PRESET_LIFT_SHADOWS,
    COLOR_PRESET_WARM_UP,
    apply_color_grade_preset,
    default_curve_data,
    default_wheels_data,
)


class ColorPresetTests(unittest.TestCase):
    def test_auto_contrast_replaces_defaults_with_mild_s_curve(self):
        payload = apply_color_grade_preset({}, COLOR_PRESET_AUTO_CONTRAST)
        self.assertEqual(payload["contrast"]["Points"][0]["co"]["Y"], 0.18)
        self.assertEqual(payload["highlights"]["Points"][0]["co"]["Y"], -0.08)
        self.assertEqual(payload["shadows"]["Points"][0]["co"]["Y"], 0.08)
        self.assertEqual(payload["curve_all"]["nodes"][1]["id"], 1)

    def test_lift_shadows_uses_fresh_neutral_channel_curves_and_wheels(self):
        payload = apply_color_grade_preset({
            "curve_red": {"enabled": False, "points": [{"x": 0.0, "y": 1.0}, {"x": 1.0, "y": 0.0}]},
            "wheels": {"enabled": False},
        }, COLOR_PRESET_LIFT_SHADOWS)
        self.assertEqual(payload["curve_red"], default_curve_data())
        self.assertEqual(payload["curve_green"], default_curve_data())
        self.assertEqual(payload["curve_blue"], default_curve_data())
        self.assertEqual(payload["wheels"], default_wheels_data())
        self.assertEqual(payload["shadows"]["Points"][0]["co"]["Y"], 0.22)

    def test_warm_up_and_boost_color_set_expected_primary_controls(self):
        warm = apply_color_grade_preset({}, COLOR_PRESET_WARM_UP)
        boost = apply_color_grade_preset({}, COLOR_PRESET_BOOST_COLOR)
        self.assertEqual(warm["temperature"]["Points"][0]["co"]["Y"], 0.18)
        self.assertEqual(warm["tint"]["Points"][0]["co"]["Y"], 0.03)
        self.assertEqual(boost["saturation"]["Points"][0]["co"]["Y"], 1.18)
        self.assertEqual(boost["vibrance"]["Points"][0]["co"]["Y"], 0.22)


if __name__ == "__main__":
    unittest.main()



"""
 @file
 @brief Painter for transition items.
 @author Jonathan Thomas <jonathan@openshot.org>

 @section LICENSE

 Copyright (c) 2008-2025 OpenShot Studios, LLC
 (http://www.openshotstudios.com). This file is part of
 OpenShot Video Editor (http://www.openshot.org), an open-source project
 dedicated to delivering high quality video editing and animation solutions
 to the world.

 OpenShot Video Editor is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 OpenShot Video Editor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
 """

from qt_api import QPointF, QRectF, Qt
from qt_api import (
    QBrush,
    QColor,
    QFontMetrics,
    QImage,
    QLinearGradient,
    QPainter,
    QPainterPath,
    QPen,
    QPixmap,
)

from .base import BasePainter
from classes.qt_types import font_metrics_horizontal_advance


class TransitionPainter(BasePainter):
    DEFAULT_OPACITY = 0.75
    SELECTED_OPACITY = 0.88
    LOCKED_OPACITY_MULTIPLIER = 0.8
    SELECTED_OVERLAY_ALPHA = 32
    MENU_PAD_X = 6.0
    MENU_PAD_Y = 2.0

    def update_theme(self):
        self.col = self.w.theme.transition.background
        self.col2 = self.w.theme.transition.background2
        bw = getattr(self.w.theme.transition, "border_width", 2.0) or 0.0
        bw = float(bw) if isinstance(bw, (int, float)) else 0.0
        if bw <= 0.0:
            bw = 2.0
        self.border_width = bw
        self.border_radius = float(self.w.theme.transition.border_radius or 0.0)
        self.pen = QPen(QBrush(self.w.theme.transition.border_color), bw)
        self.pen.setCosmetic(True)
        self.img = self.w.theme.transition.background_image
        self.sel_pen = QPen(QBrush(self.w.theme.clip_selected), bw)
        self.sel_pen.setCosmetic(True)
        from windows.views.timeline_backend.theme import _icon as _theme_icon
        arrow = _theme_icon("themes/cosmic/images/dropdown-arrow.svg")
        self.dropdown_arrow_pix = arrow if (arrow and not arrow.isNull()) else None
        self.menu_margin = self.w.theme.menu_margin
        # Cache of fully rendered transition pixmaps
        self.transition_cache = {}

    def clear_cache(self):
        """Clear cached rendered transition pixmaps."""
        self.transition_cache.clear()

    def _segment_overdraw(self, view_width):
        blur = max(0.0, float(self.w.theme.clip.shadow_blur or 0.0))
        base = max(48.0, view_width * 0.2)
        return max(base, blur * 3.0)

    def paint(self, painter: QPainter):
        area = QRectF(
            self.w.track_name_width,
            self.w.ruler_height,
            self.w.width() - self.w.track_name_width - self.w.scroll_bar_thickness,
            self.w.height() - self.w.ruler_height - self.w.scroll_bar_thickness,
        )
        overdraw = self._segment_overdraw(area.width())
        expanded = QRectF(
            area.left() - overdraw,
            area.top(),
            area.width() + (overdraw * 2.0),
            area.height(),
        )

        painter.save()
        painter.setClipRect(area)
        self.w._transition_text_rects = []
        for rect, tran, selected in self.w.geometry.iter_transitions():
            if not rect.intersects(expanded):
                continue
            segment_left = max(rect.left(), expanded.left())
            segment_right = min(rect.right(), expanded.right())
            if segment_right <= segment_left:
                continue
            segment_rect = QRectF(
                segment_left,
                rect.top(),
                segment_right - segment_left,
                rect.height(),
            )
            pen = self.sel_pen if selected else self.pen
            locked = self.w._is_track_locked((tran.data if isinstance(tran.data, dict) else {}).get("layer"))
            if locked:
                pen = self.dimmed_pen(pen)
            opacity = self.SELECTED_OPACITY if selected else self.DEFAULT_OPACITY
            if locked:
                opacity *= self.LOCKED_OPACITY_MULTIPLIER
            painter.save()
            if opacity < 0.999:
                painter.setOpacity(opacity)
            result = self._transition_pixmap(rect, segment_rect, selected=selected)
            if not result:
                painter.restore()
                continue
            pix, includes_start, includes_end = result
            if pix:
                painter.drawPixmap(segment_rect.topLeft(), pix)
            painter.restore()
            self._stroke_visible_border(
                painter,
                segment_rect,
                pen,
                includes_start=includes_start,
                includes_end=includes_end,
            )
            if includes_start:
                bw = max(float(self.border_width or 0.0), 0.0)
                title_rect = QRectF(segment_rect)
                if bw > 0.0:
                    inset_x = min(bw, max(title_rect.width() / 2.0 - 0.1, 0.0))
                    inset_y = min(bw, max(title_rect.height() / 2.0 - 0.1, 0.0))
                    title_rect.adjust(inset_x, inset_y, -inset_x, -inset_y)
                text_entry = self._draw_transition_title(
                    painter,
                    tran,
                    title_rect,
                    visible_width=float(title_rect.width()),
                )
                if isinstance(text_entry, dict):
                    self.w._transition_text_rects.append(
                        {
                            "rect": QRectF(text_entry.get("rect", QRectF())),
                            "transition": tran,
                            "title": str(text_entry.get("title", "") or ""),
                            "open_menu": bool(text_entry.get("open_menu", False)),
                        }
                    )
        painter.restore()

    def _transition_pixmap(self, full_rect, segment_rect, *, selected=False):
        """Return cached pixmap for the visible portion of a transition."""
        w = int(segment_rect.width())
        h = int(segment_rect.height())
        if w <= 0 or h <= 0:
            return None

        offset_px = max(0.0, float(segment_rect.left() - full_rect.left()))
        includes_start = offset_px <= 0.5
        includes_end = (segment_rect.right() + 0.5) >= full_rect.right()

        bg_cache_key = self.img.cacheKey() if self.img else None
        key = (w, h, bg_cache_key, round(offset_px, 2), includes_start, includes_end, bool(selected))
        if key in self.transition_cache:
            return self.transition_cache[key]

        small = w < 20
        tiny = w < 2
        radius = self.w.theme.transition.border_radius if not small else 0
        if radius:
            radius = min(float(radius), min(float(w), float(h)) / 2.0)

        img = QImage(w, h, QImage.Format_ARGB32_Premultiplied)
        img.fill(0)
        p = QPainter(img)
        p.setRenderHint(QPainter.Antialiasing, True)

        rect = QRectF(0, 0, w, h)
        path = None
        if radius > 0.0 and (includes_start or includes_end):
            left = rect.left()
            right = rect.right()
            top = rect.top()
            bottom = rect.bottom()
            path = QPainterPath()

            if includes_start:
                path.moveTo(left, top + radius)
                path.quadTo(left, top, left + radius, top)
            else:
                path.moveTo(left, top)

            if includes_end:
                path.lineTo(right - radius, top)
                path.quadTo(right, top, right, top + radius)
                path.lineTo(right, bottom - radius)
                path.quadTo(right, bottom, right - radius, bottom)
            else:
                path.lineTo(right, top)
                path.lineTo(right, bottom)

            if includes_start:
                path.lineTo(left + radius, bottom)
                path.quadTo(left, bottom, left, bottom - radius)
                path.lineTo(left, top + radius)
            else:
                path.lineTo(left, bottom)
                path.lineTo(left, top)

            path.closeSubpath()
        elif radius > 0.0:
            path = QPainterPath()
            path.addRoundedRect(rect, radius, radius)

        if not tiny:
            if self.col2.isValid() and self.col2 != self.col:
                grad = QLinearGradient(QPointF(0, 0), QPointF(0, h))
                grad.setColorAt(0, self.col)
                grad.setColorAt(1, self.col2)
                brush = QBrush(grad)
            else:
                brush = QBrush(self.col)

            if path is not None:
                p.fillPath(path, brush)
            else:
                p.fillRect(rect, brush)

            if self.img and not small:
                scaled = self.img.scaled(
                    w, h, Qt.IgnoreAspectRatio, Qt.SmoothTransformation
                )
                if path is not None:
                    p.save()
                    p.setClipPath(path)
                    p.drawPixmap(0, 0, scaled)
                    p.restore()
                else:
                    p.drawPixmap(0, 0, scaled)

            if selected:
                overlay = QColor(255, 255, 255, self.SELECTED_OVERLAY_ALPHA)
                if path is not None:
                    p.fillPath(path, overlay)
                else:
                    p.fillRect(rect, overlay)

        p.end()

        pix = QPixmap.fromImage(img)
        result = (pix, includes_start, includes_end)
        self.transition_cache[key] = result
        return result

    def _transition_title(self, tran):
        resolver = getattr(self.w, "_transition_label", None)
        if callable(resolver):
            try:
                return str(resolver(tran) or "")
            except Exception:
                pass
        data = tran.data if isinstance(getattr(tran, "data", None), dict) else {}
        return str(data.get("title", "") or "")

    def transition_menu_geometry(self, rect, tran=None, *, painter=None, visible_width=None):
        if rect.width() <= 0.0 or rect.height() <= 0.0:
            return QRectF(), QRectF(), None, QRectF(), ""

        title_raw = self._transition_title(tran) if tran is not None else ""
        metrics = QFontMetrics(painter.font()) if painter is not None else None
        font_h = float(metrics.height()) if metrics is not None else max(12.0, rect.height() - 4.0)

        pad_x = 6.0
        pad_y = 2.0
        icon_gap = 4.0
        container_h = min(rect.height(), font_h + pad_y * 2.0)
        icon_size = max(8.0, font_h - 2.0)
        compact_w = pad_x + icon_size + pad_x
        compact_lod_w = compact_w
        visible_clip_w = float(visible_width if visible_width is not None else rect.width())
        text_width = float(rect.width())

        title_elided = ""
        text_advance = 0.0
        if metrics is not None:
            avail_text_w = int(text_width - pad_x * 2.0 - icon_gap - icon_size)
            if avail_text_w >= 4:
                title_elided = metrics.elidedText(title_raw, Qt.ElideRight, avail_text_w)
                if title_elided:
                    text_advance = float(font_metrics_horizontal_advance(metrics, title_elided))

        if text_advance > 0:
            container_w = min(pad_x + text_advance + icon_gap + icon_size + pad_x, text_width)
            container_w = max(container_h, container_w)
            mode = "full"
        elif compact_w <= text_width and compact_lod_w <= visible_clip_w:
            container_w = compact_w
            mode = "compact"
        else:
            return QRectF(), QRectF(), None, QRectF(), ""

        container_rect = QRectF(rect.x(), rect.y(), container_w, max(1.0, container_h))
        scaled_arrow = None
        arrow_rect = QRectF()
        arrow_pix = self.dropdown_arrow_pix
        if arrow_pix and not arrow_pix.isNull():
            scaled_arrow = self.scaled_pixmap(arrow_pix, icon_size, icon_size)
            if scaled_arrow and not scaled_arrow.isNull():
                arrow_w, arrow_h = self.logical_size(scaled_arrow)
                if mode == "full" and text_advance > 0:
                    arrow_x = container_rect.x() + pad_x + text_advance + icon_gap
                else:
                    arrow_x = container_rect.x() + pad_x
                arrow_y = container_rect.y() + max(0.0, (container_rect.height() - arrow_h) / 2.0)
                arrow_rect = QRectF(arrow_x, arrow_y, arrow_w, arrow_h)

        return container_rect, QRectF(container_rect), scaled_arrow, arrow_rect, title_elided

    def _draw_transition_title(self, painter, tran, rect, *, visible_width=None):
        container_rect, _hit_rect, scaled_arrow, arrow_rect, title_elided = self.transition_menu_geometry(
            rect,
            tran,
            painter=painter,
            visible_width=visible_width,
        )
        if container_rect.isNull():
            return None
        radius = min(4.0, container_rect.width() / 2.0, container_rect.height() / 2.0)
        path = QPainterPath()
        path.addRoundedRect(container_rect, radius, radius)
        painter.save()
        painter.setRenderHint(QPainter.Antialiasing, True)
        painter.fillPath(path, QColor(0, 0, 0, 140))
        if title_elided:
            metrics = QFontMetrics(painter.font())
            font_h = float(metrics.height())
            pad_x = 6.0
            pad_y = 2.0
            icon_size = max(8.0, font_h - 2.0)
            icon_gap = 4.0
            text_advance = float(font_metrics_horizontal_advance(metrics, title_elided))
            flags = Qt.AlignLeft | Qt.AlignVCenter
            text_draw_rect = QRectF(
                container_rect.x() + pad_x,
                container_rect.y() + pad_y,
                text_advance,
                font_h,
            )
            painter.setPen(QColor(0, 0, 0, 120))
            painter.drawText(text_draw_rect.translated(1, 1), flags, title_elided)
            painter.setPen(self.w.theme.transition.font_color)
            painter.drawText(text_draw_rect, flags, title_elided)
        if scaled_arrow and not arrow_rect.isNull():
            painter.drawPixmap(arrow_rect.topLeft(), scaled_arrow)
        painter.restore()
        return {"rect": QRectF(container_rect), "title": self._transition_title(tran), "open_menu": True}

    def _stroke_visible_border(
        self,
        painter,
        segment_rect,
        pen,
        *,
        includes_start=True,
        includes_end=True,
    ):
        if not isinstance(pen, QPen) or not pen.color().isValid():
            return
        if segment_rect.width() <= 0.0 or segment_rect.height() <= 0.0:
            return

        painter.save()
        painter.setBrush(Qt.NoBrush)
        painter.setPen(pen)

        rect = QRectF(segment_rect)
        width_offset = max(pen.widthF(), 1.0) / 2.0
        max_x = max(rect.width() / 2.0 - 0.1, 0.0)
        max_y = max(rect.height() / 2.0 - 0.1, 0.0)
        offset_x = min(width_offset, max_x)
        offset_y = min(width_offset, max_y)
        rect.adjust(offset_x, offset_y, -offset_x, -offset_y)

        if rect.width() <= 0.0 or rect.height() <= 0.0:
            painter.restore()
            return

        radius = 0.0
        if rect.width() > 0.0 and rect.height() > 0.0:
            radius = min(self.border_radius, min(rect.width(), rect.height()) / 2.0)

        painter.setRenderHint(QPainter.Antialiasing, True)
        if radius > 0.0 and (includes_start or includes_end):
            left = rect.left()
            right = rect.right()
            top = rect.top()
            bottom = rect.bottom()
            path = QPainterPath()

            if includes_start:
                path.moveTo(left, top + radius)
                path.quadTo(left, top, left + radius, top)
            else:
                path.moveTo(left, top)

            if includes_end:
                path.lineTo(right - radius, top)
                path.quadTo(right, top, right, top + radius)
                path.lineTo(right, bottom - radius)
                path.quadTo(right, bottom, right - radius, bottom)
            else:
                path.lineTo(right, top)
                path.lineTo(right, bottom)

            if includes_start:
                path.lineTo(left + radius, bottom)
                path.quadTo(left, bottom, left, bottom - radius)
                path.lineTo(left, top + radius)
            else:
                path.lineTo(left, bottom)
                path.lineTo(left, top)

            path.closeSubpath()
            painter.drawPath(path)
        elif radius > 0.0:
            painter.drawRoundedRect(rect, radius, radius)
        else:
            painter.drawRect(rect)

        painter.restore()



"""
 @file
 @brief This file contains enums used in the timeline view (mostly for menu handling)
 @author Jonathan Thomas <jonathan@openshot.org>

 @section LICENSE

 Copyright (c) 2008-2024 OpenShot Studios, LLC
 (http://www.openshotstudios.com). This file is part of
 OpenShot Video Editor (http://www.openshot.org), an open-source project
 dedicated to delivering high quality video editing and animation solutions
 to the world.

 OpenShot Video Editor is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 OpenShot Video Editor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
 """

from enum import Enum, auto


class MenuFade(Enum):
    NONE = 0
    IN_FAST = auto()
    IN_SLOW = auto()
    OUT_FAST = auto()
    OUT_SLOW = auto()
    IN_OUT_FAST = auto()
    IN_OUT_SLOW = auto()


class MenuRotate(Enum):
    NONE = 0
    RIGHT_90 = auto()
    LEFT_90 = auto()
    FLIP_180 = auto()


class MenuLayout(Enum):
    NONE = 0
    CENTER = auto()
    TOP_LEFT = auto()
    TOP_RIGHT = auto()
    BOTTOM_LEFT = auto()
    BOTTOM_RIGHT = auto()
    ALL_WITH_ASPECT = auto()
    ALL_WITHOUT_ASPECT = auto()


class MenuAlign(Enum):
    LEFT = 0
    RIGHT = auto()


class MenuAnimate(Enum):
    NONE = 0
    # ── In ───────────────────────────────────────────────────────────────────
    SLIDE_IN_LEFT = auto()
    SLIDE_IN_RIGHT = auto()
    SLIDE_IN_TOP = auto()
    SLIDE_IN_BOTTOM = auto()
    BLUR_IN = auto()
    WIPE_IN_CIRCLE_EXPAND = auto()
    WIPE_IN_CIRCLE_SHRINK = auto()
    WIPE_IN_FADE = auto()
    WIPE_IN_LEFT = auto()
    WIPE_IN_RIGHT = auto()
    WIPE_IN_TOP = auto()
    WIPE_IN_BOTTOM = auto()
    FOCUS_WIPE_IN_CIRCLE_EXPAND = auto()
    FOCUS_WIPE_IN_CIRCLE_SHRINK = auto()
    FOCUS_WIPE_IN_LEFT = auto()
    FOCUS_WIPE_IN_RIGHT = auto()
    FOCUS_WIPE_IN_TOP = auto()
    FOCUS_WIPE_IN_BOTTOM = auto()
    POP_IN = auto()
    SPIRAL_IN = auto()
    BACK_IN_DOWN = auto()
    BACK_IN_LEFT = auto()
    BACK_IN_RIGHT = auto()
    BACK_IN_UP = auto()
    BOUNCE_IN = auto()
    BOUNCE_IN_DOWN = auto()
    BOUNCE_IN_LEFT = auto()
    BOUNCE_IN_RIGHT = auto()
    BOUNCE_IN_UP = auto()
    # ── Out ──────────────────────────────────────────────────────────────────
    SLIDE_OUT_LEFT = auto()
    SLIDE_OUT_RIGHT = auto()
    SLIDE_OUT_TOP = auto()
    SLIDE_OUT_BOTTOM = auto()
    BLUR_OUT = auto()
    WIPE_OUT_CIRCLE_EXPAND = auto()
    WIPE_OUT_CIRCLE_SHRINK = auto()
    WIPE_OUT_FADE = auto()
    WIPE_OUT_LEFT = auto()
    WIPE_OUT_RIGHT = auto()
    WIPE_OUT_TOP = auto()
    WIPE_OUT_BOTTOM = auto()
    FOCUS_WIPE_OUT_CIRCLE_EXPAND = auto()
    FOCUS_WIPE_OUT_CIRCLE_SHRINK = auto()
    FOCUS_WIPE_OUT_LEFT = auto()
    FOCUS_WIPE_OUT_RIGHT = auto()
    FOCUS_WIPE_OUT_TOP = auto()
    FOCUS_WIPE_OUT_BOTTOM = auto()
    POP_OUT = auto()
    SPIRAL_OUT = auto()
    BACK_OUT_DOWN = auto()
    BACK_OUT_LEFT = auto()
    BACK_OUT_RIGHT = auto()
    BACK_OUT_UP = auto()
    BOUNCE_OUT = auto()
    BOUNCE_OUT_DOWN = auto()
    BOUNCE_OUT_LEFT = auto()
    BOUNCE_OUT_RIGHT = auto()
    BOUNCE_OUT_UP = auto()
    # ── Emphasis ─────────────────────────────────────────────────────────────
    BOUNCE = auto()
    FLASH = auto()
    PULSE = auto()
    RUBBER_BAND = auto()
    SHAKE_X = auto()
    SHAKE_Y = auto()
    SWING = auto()
    TADA = auto()
    WOBBLE = auto()
    JELLO = auto()
    HEART_BEAT = auto()
    # ── Camera ───────────────────────────────────────────────────────────────
    CAM_PUSH_IN = auto()
    CAM_PULL_OUT = auto()
    CAM_PAN_AUTO = auto()
    CAM_PAN_LEFT = auto()
    CAM_PAN_RIGHT = auto()
    CAM_PAN_UP = auto()
    CAM_PAN_DOWN = auto()
    KEN_BURNS_IN = auto()
    KEN_BURNS_OUT = auto()
    KEN_BURNS_IN_LEFT_TO_RIGHT = auto()
    KEN_BURNS_IN_RIGHT_TO_LEFT = auto()
    KEN_BURNS_IN_TOP_TO_BOTTOM = auto()
    KEN_BURNS_IN_BOTTOM_TO_TOP = auto()
    KEN_BURNS_OUT_LEFT_TO_RIGHT = auto()
    KEN_BURNS_OUT_RIGHT_TO_LEFT = auto()
    KEN_BURNS_OUT_TOP_TO_BOTTOM = auto()
    KEN_BURNS_OUT_BOTTOM_TO_TOP = auto()
    # ── Credits ──────────────────────────────────────────────────────────────
    CREDITS_UP = auto()
    CREDITS_DOWN = auto()


class MenuVolume(Enum):
    NONE = 1
    FADE_IN_FAST = auto()
    FADE_IN_SLOW = auto()
    FADE_OUT_FAST = auto()
    FADE_OUT_SLOW = auto()
    FADE_IN_OUT_FAST = auto()
    FADE_IN_OUT_SLOW = auto()
    LEVEL = auto()


class MenuTime(Enum):
    NONE = 0
    FORWARD = auto()
    BACKWARD = auto()
    REVERSE = auto()
    FREEZE = auto()
    FREEZE_ZOOM = auto()


class MenuCopy(Enum):
    ALL = -1
    CLIP = 0
    KEYFRAMES_ALL = auto()
    KEYFRAMES_ALPHA = auto()
    KEYFRAMES_SCALE = auto()
    KEYFRAMES_SHEAR = auto()
    KEYFRAMES_ROTATE = auto()
    KEYFRAMES_LOCATION = auto()
    KEYFRAMES_TIME = auto()
    KEYFRAMES_VOLUME = auto()
    EFFECT = auto()
    ALL_EFFECTS = auto()
    PASTE = auto()
    TRANSITION = auto()
    KEYFRAMES_BRIGHTNESS = auto()
    KEYFRAMES_CONTRAST = auto()


class MenuSlice(Enum):
    KEEP_BOTH = 0
    KEEP_LEFT = auto()
    KEEP_RIGHT = auto()


class MenuSplitAudio(Enum):
    SINGLE = 0
    MULTIPLE = auto()



"""
 @file
 @brief This file creates a styled QMenu (which supports border radius and border color)
 @author Jonathan Thomas <jonathan@openshot.org>

 @section LICENSE

 Copyright (c) 2008-2024 OpenShot Studios, LLC
 (http://www.openshotstudios.com). This file is part of
 OpenShot Video Editor (http://www.openshot.org), an open-source project
 dedicated to delivering high quality video editing and animation solutions
 to the world.

 OpenShot Video Editor is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 OpenShot Video Editor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
 """

import logging
import re
import functools

from qt_api import QMenu, isdeleted
from qt_api import QPainter, QPen, QColor
from qt_api import Qt, QRectF
from classes.app import get_app


logger = logging.getLogger(__name__)


class StyledContextMenu(QMenu):
    def __init__(self, title=None, parent=None):
        super().__init__(title, parent)
        self.app = get_app()
        self.border = self.get_border()
        self.border_radius = self.get_border_radius()

    def show_at(self, event_or_pos):
        """Show the menu at a position or context menu event."""
        pos = event_or_pos
        if hasattr(event_or_pos, "globalPosition"):
            try:
                pos = event_or_pos.globalPosition().toPoint()
            except Exception:
                pos = event_or_pos
        if hasattr(event_or_pos, "globalPos"):
            try:
                pos = event_or_pos.globalPos()
            except Exception:
                pos = event_or_pos
        exec_fn = getattr(self, "exec", None) or getattr(self, "exec_", None)
        if exec_fn:
            exec_fn(pos)
        else:
            self.popup(pos)

    def get_border(self):
        """Parses border width and color from app.styleSheet()"""
        pattern = r'QMenu\s*{\s*[^}]*border:\s*([^;]+);'
        match = re.search(pattern, self.app.styleSheet(), re.IGNORECASE)
        if match:
            border_parts = match.group(1).split()
            # Typically, border is defined as width style color
            if len(border_parts) >= 3:
                width = float(border_parts[0].replace('px', ''))  # Remove 'px' and convert to float
                style = border_parts[1]
                color = QColor(border_parts[2])
                return {'width': width, 'style': style, 'color': color}
        return None

    def get_border_radius(self):
        """Parses border radius from app.styleSheet()"""
        pattern = r'QMenu\s*{\s*[^}]*border-radius:\s*([^;]+);'
        match = re.search(pattern, self.app.styleSheet(), re.IGNORECASE)
        if match:
            # Split the radius values by whitespace and remove 'px' unit
            radius_values = match.group(1).replace('px', '').split()
            if len(radius_values) == 1:
                radius = float(radius_values[0])
                return {'x': radius, 'y': radius}
            if len(radius_values) == 2:
                return {'x': float(radius_values[0]), 'y': float(radius_values[1])}
        return None

    def paintEvent(self, event):
        """Paint optional borders and corner radius on QMenu"""
        super().paintEvent(event)

        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)

        if self.border:
            pen = QPen(self.border.get('color'), self.border.get('width'))
            painter.setPen(pen)

        if self.border_radius:
            rect = QRectF(0, 0, self.width(), self.height())
            painter.drawRoundedRect(rect, self.border_radius.get('x'), self.border_radius.get('y'), Qt.AbsoluteSize)

        painter.end()


def _resolve_live_owner(owner):
    """Resolve a live QObject owner for menu action metadata/trigger lookup."""
    if owner is None:
        return get_app().window
    try:
        if not isdeleted(owner):
            return owner
    except Exception:
        return get_app().window
    return get_app().window


def add_bound_action(menu, owner, action_name, fallback_text, callback=None, enabled=None):
    """Add a fresh menu-owned action and resolve the current slot at trigger time."""
    owner = _resolve_live_owner(owner)
    source_action = getattr(owner, action_name, None)
    label = fallback_text
    icon = None
    if source_action is not None:
        try:
            if not isdeleted(source_action):
                label = source_action.text() or fallback_text
                icon = source_action.icon()
                if enabled is None:
                    enabled = source_action.isEnabled()
        except Exception as exc:
            logger.debug("menu: failed to read source action metadata for %s: %s", action_name, exc, exc_info=True)
    action = menu.addAction(icon, label) if icon and not icon.isNull() else menu.addAction(label)
    if enabled is not None:
        action.setEnabled(bool(enabled))

    def trigger_slot():
        live_owner = _resolve_live_owner(owner)
        live_callback = callback
        if isinstance(callback, str):
            live_callback = getattr(live_owner, callback, None)
        if live_callback is None:
            live_callback = getattr(live_owner, f"{action_name}_trigger", None)
        if live_callback is None:
            raise AttributeError(f"Unable to resolve callback for {action_name}")
        live_callback()

    action.triggered.connect(lambda checked=False: trigger_slot())
    return action


def keyframe_bezier_presets():
    _ = get_app()._tr
    return [
        (0.250, 0.100, 0.250, 1.000, _("Ease (Default)")),
        (0.420, 0.000, 1.000, 1.000, _("Ease In")),
        (0.000, 0.000, 0.580, 1.000, _("Ease Out")),
        (0.420, 0.000, 0.580, 1.000, _("Ease In/Out")),
        (0.550, 0.085, 0.680, 0.530, _("Ease In (Quad)")),
        (0.550, 0.055, 0.675, 0.190, _("Ease In (Cubic)")),
        (0.895, 0.030, 0.685, 0.220, _("Ease In (Quart)")),
        (0.755, 0.050, 0.855, 0.060, _("Ease In (Quint)")),
        (0.470, 0.000, 0.745, 0.715, _("Ease In (Sine)")),
        (0.950, 0.050, 0.795, 0.035, _("Ease In (Expo)")),
        (0.600, 0.040, 0.980, 0.335, _("Ease In (Circ)")),
        (0.600, -0.280, 0.735, 0.045, _("Ease In (Back)")),
        (0.250, 0.460, 0.450, 0.940, _("Ease Out (Quad)")),
        (0.215, 0.610, 0.355, 1.000, _("Ease Out (Cubic)")),
        (0.165, 0.840, 0.440, 1.000, _("Ease Out (Quart)")),
        (0.230, 1.000, 0.320, 1.000, _("Ease Out (Quint)")),
        (0.390, 0.575, 0.565, 1.000, _("Ease Out (Sine)")),
        (0.190, 1.000, 0.220, 1.000, _("Ease Out (Expo)")),
        (0.075, 0.820, 0.165, 1.000, _("Ease Out (Circ)")),
        (0.175, 0.885, 0.320, 1.275, _("Ease Out (Back)")),
        (0.455, 0.030, 0.515, 0.955, _("Ease In/Out (Quad)")),
        (0.645, 0.045, 0.355, 1.000, _("Ease In/Out (Cubic)")),
        (0.770, 0.000, 0.175, 1.000, _("Ease In/Out (Quart)")),
        (0.860, 0.000, 0.070, 1.000, _("Ease In/Out (Quint)")),
        (0.445, 0.050, 0.550, 0.950, _("Ease In/Out (Sine)")),
        (1.000, 0.000, 0.000, 1.000, _("Ease In/Out (Expo)")),
        (0.785, 0.135, 0.150, 0.860, _("Ease In/Out (Circ)")),
        (0.680, -0.550, 0.265, 1.550, _("Ease In/Out (Back)")),
    ]


def populate_keyframe_context_menu(
    menu,
    *,
    bezier_callback,
    linear_callback,
    constant_callback,
    remove_callback=None,
    remove_enabled=True,
    bezier_icon=None,
    linear_icon=None,
    constant_icon=None,
    remove_text=None,
):
    _ = get_app()._tr

    bez_menu = menu.addMenu(bezier_icon, _("Bezier")) if bezier_icon else menu.addMenu(_("Bezier"))
    for preset in keyframe_bezier_presets():
        action = bez_menu.addAction(preset[4])
        action.triggered.connect(functools.partial(bezier_callback, preset))

    linear_action = menu.addAction(linear_icon, _("Linear")) if linear_icon else menu.addAction(_("Linear"))
    linear_action.triggered.connect(linear_callback)

    constant_action = menu.addAction(constant_icon, _("Constant")) if constant_icon else menu.addAction(_("Constant"))
    constant_action.triggered.connect(constant_callback)

    if remove_callback is not None:
        menu.addSeparator()
        remove_action = menu.addAction(remove_text or _("Remove Keyframe"))
        remove_action.setEnabled(bool(remove_enabled))
        remove_action.triggered.connect(remove_callback)
        return remove_action
    return None



"""
 @file
 @brief This file contains unit tests for VideoWidget transform and location geometry
 @author Jonathan Thomas <jonathan@openshot.org>

 @section LICENSE

 Copyright (c) 2008-2026 OpenShot Studios, LLC
 (http://www.openshotstudios.com). This file is part of
 OpenShot Video Editor (http://www.openshot.org), an open-source project
 dedicated to delivering high quality video editing and animation solutions
 to the world.

 OpenShot Video Editor is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 OpenShot Video Editor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
 """

import os
import sys
import tempfile
import types
import unittest
from unittest.mock import patch

import openshot
from qt_api import QApplication, QColor, QLabel, QPoint, QPointF, QPushButton, QRect, QRectF, QSize, QStandardItem, QTransform, Qt, QWidget


PATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
if PATH not in sys.path:
    sys.path.append(PATH)

from tests.qt_test_app import ensure_app_state, get_or_create_app


class DummySettings:
    def __init__(self):
        self.values = {}

    def get(self, key):
        return self.values.get(key, False)

    def set(self, key, value):
        self.values[key] = value


class DummyApp(QApplication):
    def __init__(self):
        super().__init__([])
        self.settings = DummySettings()


app, _owns_app = get_or_create_app(DummyApp)
ensure_app_state(app, DummySettings, extra_attrs={"window": types.SimpleNamespace()})

from windows.video_widget import VideoWidget
from windows.models.properties_model import ClipStandardItemModel, PropertiesModel
from windows.process_effect import (
    ProcessEffect,
    CUTIE_MODELS_PATH,
    EFFICIENT_SAM_MODELS_PATH,
    compact_model_label,
    load_model_manifest,
    load_yolo_models_manifest,
    object_mask_quality_label,
    yolo_classes_path,
    yolo_download_button_label,
    yolo_model_label,
    yolo_model_path,
)
from classes import http_client
from classes.effect_init import effect_options


def clip_with(scale_mode, gravity=openshot.GRAVITY_CENTER):
    return types.SimpleNamespace(data={"scale": scale_mode, "gravity": gravity})


def props(location_x=0.0, location_y=0.0, scale_x=1.0, scale_y=1.0, margin=0.0):
    return {
        "scale_x": {"value": scale_x},
        "scale_y": {"value": scale_y},
        "rotation": {"value": 0.0},
        "shear_x": {"value": 0.0},
        "shear_y": {"value": 0.0},
        "origin_x": {"value": 0.5},
        "origin_y": {"value": 0.5},
        "location_x": {"value": location_x},
        "location_y": {"value": location_y},
        "margin": {"value": margin},
        "parentObjectId": {"memo": ""},
    }


class FakeSignal:
    def __init__(self):
        self.count = 0

    def emit(self, *args):
        self.count += 1


def mouse_event_at(x, y):
    return types.SimpleNamespace(pos=lambda: QPoint(x, y))


class FakePropertiesParent:
    def __init__(self, model):
        self.model = model

    def currentIndex(self):
        return self.model.index(0, 1)

    def clearSelection(self):
        pass

    def setCurrentIndex(self, index):
        pass


class VideoWidgetTransformTests(unittest.TestCase):
    def setUp(self):
        self.widget = VideoWidget.__new__(VideoWidget)
        self.viewport = QRect(0, 0, 160, 90)

    def rect_for(self, scale_mode, location_x=0.0, location_y=0.0, scale_x=1.0, scale_y=1.0):
        return VideoWidget._clip_display_rect(
            self.widget,
            40,
            40,
            clip_with(scale_mode),
            props(location_x, location_y, scale_x, scale_y),
            self.viewport,
        )

    def test_square_clip_location_y_endpoints_are_offscreen_for_fit_and_crop(self):
        for scale_mode in (openshot.SCALE_FIT, openshot.SCALE_CROP):
            with self.subTest(scale_mode=scale_mode):
                top = self.rect_for(scale_mode, location_y=-1.0)
                bottom = self.rect_for(scale_mode, location_y=1.0)

                self.assertLessEqual(top.y() + top.height(), 0.0)
                self.assertGreaterEqual(bottom.y(), self.viewport.height())

    def test_square_clip_location_x_endpoints_are_offscreen_for_fit_and_crop(self):
        for scale_mode in (openshot.SCALE_FIT, openshot.SCALE_CROP):
            with self.subTest(scale_mode=scale_mode):
                left = self.rect_for(scale_mode, location_x=-1.0)
                right = self.rect_for(scale_mode, location_x=1.0)

                self.assertLessEqual(left.x() + left.width(), 0.0)
                self.assertGreaterEqual(right.x(), self.viewport.width())

    def test_clip_margin_offsets_edge_gravity_handle_rect(self):
        project_values = {"width": 160, "height": 90}
        app = types.SimpleNamespace(project=types.SimpleNamespace(
            get=project_values.get))
        with patch("windows.video_widget.get_app", return_value=app):
            rect = VideoWidget._clip_display_rect(
                self.widget,
                40,
                30,
                clip_with(openshot.SCALE_NONE, openshot.GRAVITY_BOTTOM_RIGHT),
                props(scale_x=1.0, scale_y=1.0, margin=0.1),
                self.viewport,
            )

        self.assertAlmostEqual(rect.x(), 111.0)
        self.assertAlmostEqual(rect.y(), 51.0)
        self.assertAlmostEqual(rect.width(), 40.0)
        self.assertAlmostEqual(rect.height(), 30.0)

    def test_clip_margin_reduces_fit_layout_handle_rect(self):
        rect = VideoWidget._clip_display_rect(
            self.widget,
            160,
            90,
            clip_with(openshot.SCALE_FIT, openshot.GRAVITY_CENTER),
            props(margin=0.1),
            self.viewport,
        )

        self.assertAlmostEqual(rect.x(), 16.0)
        self.assertAlmostEqual(rect.y(), 9.0)
        self.assertAlmostEqual(rect.width(), 128.0)
        self.assertAlmostEqual(rect.height(), 72.0)

    def test_clip_screen_rect_applies_clip_scale_before_group_union(self):
        first = VideoWidget._clip_screen_rect(
            self.widget,
            QRectF(0.0, 0.0, 160.0, 90.0),
            props(scale_x=0.5, scale_y=0.5),
        )
        second = VideoWidget._clip_screen_rect(
            self.widget,
            QRectF(80.0, 0.0, 160.0, 90.0),
            props(scale_x=0.5, scale_y=0.5),
        )
        union = first.united(second)

        self.assertAlmostEqual(first.width(), 80.0)
        self.assertAlmostEqual(second.x(), 80.0)
        self.assertAlmostEqual(union.width(), 160.0)

    def test_yolo5_file_sha256(self):
        with tempfile.NamedTemporaryFile(delete=False) as test_file:
            test_file.write(b"openshot-yolov5")
            test_path = test_file.name
        try:
            self.assertEqual(
                ProcessEffect.file_sha256(None, test_path),
                "c54dd3b21ba1b2283d358605d1c7740ce50adc337b4682f5d96c954db4390337",
            )
        finally:
            os.remove(test_path)

    def test_yolo_manifest_has_packaged_recommended_default(self):
        manifest = load_yolo_models_manifest()
        recommended = [model for model in manifest["models"] if model.get("recommended")]

        self.assertEqual(len(recommended), 1)
        self.assertEqual(recommended[0]["id"], "yolo26n-seg")
        self.assertEqual(yolo_model_label(recommended[0]), "YOLO26: Nano (recommended, fast)")
        self.assertEqual(yolo_download_button_label(recommended[0]), "Download (10 MB)")
        self.assertTrue(yolo_model_path(recommended[0]).endswith("yolo26n-seg/model.onnx"))
        self.assertTrue(yolo_classes_path(recommended[0]).endswith("yolo26n-seg/classes.names"))

    def test_ai_model_manifests_use_shared_catalog_shape(self):
        for manifest in (
            load_yolo_models_manifest(),
            load_model_manifest(EFFICIENT_SAM_MODELS_PATH),
            load_model_manifest(CUTIE_MODELS_PATH),
        ):
            self.assertEqual(set(manifest.keys()), {"version", "release", "base_url", "models"})
            self.assertEqual(manifest["release"], "v0.2.0")
            for model in manifest["models"]:
                self.assertLessEqual(
                    set(model.keys()),
                    {"id", "name", "description", "asset", "sha256", "bytes", "recommended"},
                )

    def test_ai_model_manifests_use_ui_copy(self):
        yolo_models = load_yolo_models_manifest()["models"]
        cutie_models = load_model_manifest(CUTIE_MODELS_PATH)["models"]
        efficient_sam_models = load_model_manifest(EFFICIENT_SAM_MODELS_PATH)["models"]

        self.assertNotIn("yolov8n-seg", {model["id"] for model in yolo_models})
        self.assertEqual(yolo_models[0]["description"], "recommended, fast")
        self.assertEqual(yolo_models[1]["description"], "balanced")
        self.assertEqual(yolo_models[2]["description"], "quality")
        self.assertNotIn("cutie-very-high", {model["id"] for model in cutie_models})
        self.assertEqual(
            [(model["name"], model["description"]) for model in cutie_models],
            [
                ("Small", "fast"),
                ("Medium", "recommended, balanced"),
                ("Large", "quality"),
            ],
        )
        self.assertEqual(efficient_sam_models[0]["description"], "recommended, fast")

    def test_yolo5_validation_uses_libopenshot(self):
        process = ProcessEffect.__new__(ProcessEffect)
        process.onnx_validation_cache = {}

        with tempfile.NamedTemporaryFile(suffix=".onnx") as test_model, \
                patch("windows.process_effect.get_app") as get_app, \
                patch(
                    "windows.process_effect.openshot.ClipProcessingJobs.ValidateONNXModel",
                    return_value="",
                ) as validate:
            get_app.return_value = types.SimpleNamespace(_tr=lambda text: text)

            valid, message = ProcessEffect.validate_onnx_model_load(process, test_model.name)

        self.assertTrue(valid)
        self.assertEqual(message, "Ready")
        validate.assert_called_once_with(test_model.name)

    def test_yolo5_validation_reports_libopenshot_error(self):
        process = ProcessEffect.__new__(ProcessEffect)
        process.onnx_validation_cache = {}

        with tempfile.NamedTemporaryFile(suffix=".onnx") as test_model, \
                patch("windows.process_effect.get_app") as get_app, \
                patch(
                    "windows.process_effect.openshot.ClipProcessingJobs.ValidateONNXModel",
                    return_value="Failed to load ONNX model: bad graph",
                ):
            get_app.return_value = types.SimpleNamespace(_tr=lambda text: text)

            valid, message = ProcessEffect.validate_onnx_model_load(process, test_model.name)

        self.assertFalse(valid)
        self.assertEqual(message, "Failed to load ONNX model: bad graph")

    def test_object_detection_generate_masks_option_is_not_shown(self):
        generate_masks = [
            option for option in effect_options["ObjectDetection"]
            if option.get("setting") == "generate_masks"
        ]

        self.assertEqual(generate_masks, [])

    def test_object_detection_model_paths_are_advanced(self):
        object_detection_options = effect_options["ObjectDetection"]
        download = next(option for option in object_detection_options if option.get("setting") == "download-yolo")
        model = next(option for option in object_detection_options if option.get("setting") == "model")
        classes = next(option for option in object_detection_options if option.get("setting") == "classes_file")
        device = next(option for option in object_detection_options if option.get("setting") == "processing-device")

        self.assertEqual(download["title"], "Version")
        self.assertEqual(download["file-settings"], ["model", "classes_file"])
        self.assertTrue(model["advanced"])
        self.assertTrue(classes["advanced"])
        self.assertTrue(device["advanced"])
        self.assertEqual(device["value"], "CPU")

    def test_object_mask_initializer_defaults_to_efficientsam_downloads(self):
        object_mask_options = effect_options["ObjectMask"]
        download = next(option for option in object_mask_options if option.get("setting") == "download-object-mask")
        efficient_sam = next(option for option in object_mask_options if option.get("setting") == "efficient_sam_model")
        cutie_key = next(option for option in object_mask_options if option.get("setting") == "cutie_encode_key_model")
        cutie_value = next(option for option in object_mask_options if option.get("setting") == "cutie_encode_value_model")
        cutie_readout = next(option for option in object_mask_options if option.get("setting") == "cutie_memory_readout_model")
        cutie_decode = next(option for option in object_mask_options if option.get("setting") == "cutie_decode_model")
        selector = next(option for option in object_mask_options if option.get("setting") == "object_mask_selection")
        device = next(option for option in object_mask_options if option.get("setting") == "processing-device")
        selector_index = object_mask_options.index(selector)
        device_index = object_mask_options.index(device)

        self.assertEqual(download["type"], "download-object-mask")
        self.assertEqual(download["title"], "Quality")
        self.assertEqual(download["file-settings"], [
            "efficient_sam_model",
            "cutie_encode_key_model",
            "cutie_encode_value_model",
            "cutie_memory_readout_model",
            "cutie_decode_model",
        ])
        self.assertEqual(efficient_sam["type"], "file")
        self.assertEqual(cutie_key["type"], "file")
        self.assertEqual(cutie_value["type"], "file")
        self.assertEqual(cutie_readout["type"], "file")
        self.assertEqual(cutie_decode["type"], "file")
        self.assertTrue(efficient_sam["advanced"])
        self.assertTrue(cutie_key["advanced"])
        self.assertTrue(cutie_value["advanced"])
        self.assertTrue(cutie_readout["advanced"])
        self.assertTrue(cutie_decode["advanced"])
        self.assertTrue(device["advanced"])
        self.assertEqual(device["value"], "CPU")
        self.assertLess(device_index, selector_index)
        self.assertTrue(efficient_sam["value"].endswith(
            ".openshot_qt/yolo/efficient-sam-tiny-1024/image_segmentation_efficientsam_ti_2025april.onnx"))
        self.assertTrue(cutie_key["value"].endswith(".openshot_qt/yolo/cutie-medium/cutie-encode-key-640x368.onnx"))
        self.assertTrue(cutie_value["value"].endswith(".openshot_qt/yolo/cutie-medium/cutie-encode-value-640x368.onnx"))
        self.assertTrue(cutie_readout["value"].endswith(
            ".openshot_qt/yolo/cutie-medium/cutie-memory-readout-floatmask-valid-640x368-m6-topk30-opencv.onnx"))
        self.assertTrue(cutie_decode["value"].endswith(".openshot_qt/yolo/cutie-medium/cutie-decode-640x368.onnx"))
        self.assertEqual(selector["type"], "object-mask-selection")
        self.assertEqual(selector["title"], "Select Points")

    def test_cutie_quality_dropdown_uses_compact_label(self):
        manifest = load_model_manifest(CUTIE_MODELS_PATH)
        recommended = next(model for model in manifest["models"] if model.get("recommended"))

        self.assertEqual(compact_model_label(recommended), "Medium")
        self.assertEqual(object_mask_quality_label(recommended), "Medium (recommended, balanced)")

    def test_object_mask_selector_is_disabled_until_models_are_ready(self):
        process = ProcessEffect.__new__(ProcessEffect)
        process.download_groups = [{"type": "download-object-mask", "ready": False}]
        button = QPushButton()
        status = QLabel()
        process.selection_fields = {
            "object_mask_selection": {
                "button": button,
                "status": status,
                "valid": False,
            }
        }

        with patch("windows.process_effect.get_app") as get_app:
            get_app.return_value = types.SimpleNamespace(_tr=lambda text: text)
            valid = ProcessEffect.update_selection_validation(process)

        self.assertFalse(valid)
        self.assertFalse(button.isEnabled())
        self.assertEqual(status.toolTip(), "Object Mask model files are not ready.")

    def test_object_mask_partial_archive_blocks_ready_status(self):
        process = ProcessEffect.__new__(ProcessEffect)
        status = QLabel()
        process.download_groups = [{
            "type": "download-object-mask",
            "file-settings": ["efficient_sam_model", "cutie_encode_key_model"],
            "status": status,
        }]
        process.selected_download_group_model = lambda group: {"id": "cutie-medium"}
        process.efficient_sam_model_by_id = lambda: {"id": "efficient-sam-tiny-1024"}
        file_results = {
            "efficient_sam_model": {
                "valid": True,
                "message": "Valid model file",
                "path": "/tmp/image_segmentation_efficientsam_ti_2025april.onnx",
            },
            "cutie_encode_key_model": {
                "valid": True,
                "message": "Valid model file",
                "path": "/tmp/cutie-encode-key-640x368.onnx",
            },
        }

        with tempfile.TemporaryDirectory() as sam_dir, tempfile.TemporaryDirectory() as cutie_dir:
            open(os.path.join(cutie_dir, "cutie-medium.zip"), "wb").close()
            with patch("windows.process_effect.get_app") as get_app, \
                    patch("windows.process_effect.model_install_dir") as install_dir:
                get_app.return_value = types.SimpleNamespace(_tr=lambda text: text)
                install_dir.side_effect = (
                    lambda model: sam_dir
                    if str(model.get("id", "")).startswith("efficient-sam")
                    else cutie_dir
                )
                ProcessEffect.update_download_group_statuses(process, file_results)

        self.assertFalse(process.download_groups[0]["ready"])
        self.assertEqual(status.text(), "Not Downloaded")
        self.assertIn("incomplete", status.toolTip())

    def test_object_mask_payload_converts_to_preprocess_context(self):
        payload = {
            "seed_frame": 1,
            "frames": {
                "1": {
                    "positive_points": [{"x": 375.0, "y": 175.0}],
                    "negative_points": [{"x": 295.0, "y": 65.0}, {"x": 315.0, "y": 75.0}],
                    "positive_rects": [{"x1": 300.0, "y1": 120.0, "x2": 420.0, "y2": 240.0}],
                }
            },
        }

        context, valid = ProcessEffect.object_mask_context_from_payload(payload)

        self.assertTrue(valid)
        self.assertEqual(context["positive_x"], 375.0)
        self.assertEqual(context["positive_y"], 175.0)
        self.assertEqual(context["negative_x"], 295.0)
        self.assertEqual(context["negative_y"], 65.0)
        self.assertEqual(len(context["negative_points"]), 2)
        self.assertEqual(context["negative_points"][1]["x"], 315.0)
        self.assertEqual(context["rect_x1"], 300.0)
        self.assertEqual(context["rect_y1"], 120.0)
        self.assertEqual(context["rect_x2"], 420.0)
        self.assertEqual(context["rect_y2"], 240.0)

    def test_object_mask_rect_payload_uses_center_as_positive_prompt(self):
        payload = {
            "seed_frame": 1,
            "frames": {
                "1": {
                    "positive_rects": [{"x1": 10.0, "y1": 20.0, "x2": 30.0, "y2": 60.0}],
                }
            },
        }

        context, valid = ProcessEffect.object_mask_context_from_payload(payload)

        self.assertTrue(valid)
        self.assertEqual(context["positive_x"], 20.0)
        self.assertEqual(context["positive_y"], 40.0)

    def test_object_mask_payload_scales_preview_pixels_to_source_pixels(self):
        payload = {
            "seed_frame": 1,
            "frames": {
                "1": {
                    "positive_points": [{"x": 290.0, "y": 136.0}],
                    "negative_points": [{"x": 227.0, "y": 50.0}],
                    "positive_rects": [{"x1": 100.0, "y1": 20.0, "x2": 200.0, "y2": 80.0}],
                },
                "300": {
                    "positive_points": [{"x": 320.0, "y": 140.0}],
                }
            },
        }
        preview_size = types.SimpleNamespace(width=lambda: 495, height=lambda: 280)
        source_size = types.SimpleNamespace(width=lambda: 640, height=lambda: 360)

        scaled = ProcessEffect.object_mask_payload_to_source_pixels(payload, preview_size, source_size)

        frame = scaled["frames"]["1"]
        self.assertAlmostEqual(frame["positive_points"][0]["x"], 374.949, places=3)
        self.assertAlmostEqual(frame["positive_points"][0]["y"], 174.857, places=3)
        self.assertAlmostEqual(frame["negative_points"][0]["x"], 293.495, places=3)
        self.assertAlmostEqual(frame["negative_points"][0]["y"], 64.286, places=3)
        self.assertAlmostEqual(frame["positive_rects"][0]["x1"], 129.293, places=3)
        self.assertAlmostEqual(frame["positive_rects"][0]["y2"], 102.857, places=3)
        self.assertAlmostEqual(scaled["frames"]["300"]["positive_points"][0]["x"], 413.737, places=3)

    def test_display_frame_size_ignores_partial_paint_rect_size(self):
        frame_size = VideoWidget._scaled_frame_size(QSize(810, 456), QSize(810, 456))

        self.assertEqual(frame_size.width(), 810)
        self.assertEqual(frame_size.height(), 456)

    def test_region_rect_edges_can_reach_frame_bounds(self):
        self.widget.curr_frame_size = QSize(640, 360)

        point = VideoWidget._clamp_region_point(self.widget, QPointF(640.0, 360.0))
        edge = VideoWidget._clamp_region_point(self.widget, QPointF(640.0, 360.0), include_edges=True)

        self.assertEqual(point, QPointF(639.0, 359.0))
        self.assertEqual(edge, QPointF(640.0, 360.0))

    def test_process_effect_disables_editable_controls_while_processing(self):
        process = ProcessEffect.__new__(ProcessEffect)
        enabled_calls = {
            "scroll": [],
            "process": [],
            "cancel": [],
        }
        process.scrollArea = types.SimpleNamespace(setEnabled=enabled_calls["scroll"].append)
        process.process_button = types.SimpleNamespace(setEnabled=enabled_calls["process"].append)
        process.cancel_button = types.SimpleNamespace(setEnabled=enabled_calls["cancel"].append)

        ProcessEffect.set_processing_controls_enabled(process, False)

        self.assertEqual(enabled_calls["scroll"], [False])
        self.assertEqual(enabled_calls["process"], [False])
        self.assertEqual(enabled_calls["cancel"], [True])

    def test_process_effect_cancel_tolerates_invalidated_native_job(self):
        class InvalidatedProcessingJob:
            def __init__(self):
                self.cancel_calls = 0
                self.done_calls = 0

            def CancelProcessing(self):
                self.cancel_calls += 1

            def IsDone(self):
                self.done_calls += 1
                raise RuntimeError("Invalid argument")

        processing = InvalidatedProcessingJob()

        ProcessEffect.cancel_processing_job(processing)

        self.assertEqual(processing.cancel_calls, 1)
        self.assertEqual(processing.done_calls, 1)

    def test_yolo5_ssl_context_prefers_certifi_bundle(self):
        certifi_stub = types.SimpleNamespace(where=lambda: "/tmp/cacert.pem")
        context_stub = object()

        with patch.dict(sys.modules, {"certifi": certifi_stub}), \
                patch("classes.http_client.os.path.exists", return_value=True), \
                patch("classes.http_client.ssl.create_default_context", return_value=context_stub) as create_context:
            self.assertIs(http_client.ssl_context(), context_stub)

        create_context.assert_called_once_with(cafile="/tmp/cacert.pem")

    def test_download_url_to_file_streams_with_ssl_context_and_progress(self):
        class FakeResponse:
            headers = {"Content-Length": "11"}

            def __init__(self):
                self.chunks = [b"hello ", b"world", b""]

            def __enter__(self):
                return self

            def __exit__(self, *_args):
                return False

            def read(self, _size):
                return self.chunks.pop(0)

        progress = []
        context_stub = object()
        response = FakeResponse()

        with tempfile.NamedTemporaryFile(delete=False) as test_file:
            test_path = test_file.name
        try:
            with patch("classes.http_client.ssl_context", return_value=context_stub), \
                    patch("classes.http_client.urllib.request.urlopen", return_value=response) as urlopen:
                http_client.download_file(
                    "https://example.com/model.zip",
                    test_path,
                    "test model",
                    lambda downloaded, total: progress.append((downloaded, total)),
                )

            with open(test_path, "rb") as output_file:
                self.assertEqual(output_file.read(), b"hello world")
            self.assertEqual(progress, [(6, 11), (11, 11)])
            self.assertIs(urlopen.call_args.kwargs["context"], context_stub)
        finally:
            os.remove(test_path)

    def test_location_offset_inverse_round_trips_drag_motion(self):
        # Crop square in a 16:9 viewport renders as 160x160, centered at y=-35.
        source_w, source_h, scaled_w, scaled_h, anchor_x, anchor_y, layout_x, layout_y, layout_w, layout_h = (
            VideoWidget._clip_location_geometry(
                self.widget,
                40,
                40,
                clip_with(openshot.SCALE_CROP),
                props(),
                self.viewport,
            )
        )
        self.assertEqual(
            (source_w, source_h, scaled_w, scaled_h, anchor_x, anchor_y, layout_x, layout_y, layout_w, layout_h),
            (160.0, 160.0, 160.0, 160.0, 0.0, -35.0, 0.0, 0.0, 160.0, 90.0))

        for location in (-1.0, -0.5, 0.0, 0.5, 1.0):
            with self.subTest(location=location):
                offset = VideoWidget._location_offset(location, anchor_y, self.viewport.height(), scaled_h)
                restored = VideoWidget._location_value_from_offset(
                    offset, anchor_y, self.viewport.height(), scaled_h)
                self.assertAlmostEqual(restored, location, places=6)

    def test_scale_none_uses_project_to_viewport_pixel_ratio(self):
        fake_app = types.SimpleNamespace(
            project=types.SimpleNamespace(get={"width": 320, "height": 180}.get)
        )
        with patch("windows.video_widget.get_app", return_value=fake_app):
            center = self.rect_for(openshot.SCALE_NONE)
            self.assertAlmostEqual(center.width(), 20.0)
            self.assertAlmostEqual(center.height(), 20.0)
            self.assertAlmostEqual(center.x(), 70.0)
            self.assertAlmostEqual(center.y(), 35.0)

            top = self.rect_for(openshot.SCALE_NONE, location_y=-1.0)
            bottom = self.rect_for(openshot.SCALE_NONE, location_y=1.0)
            self.assertLessEqual(top.y() + top.height(), 0.0)
            self.assertGreaterEqual(bottom.y(), self.viewport.height())

    def test_margin_box_norm_converts_effect_margins_to_region(self):
        raw = {
            "left": {"value": 0.10},
            "top": {"value": 0.20},
            "right": {"value": 0.30},
            "bottom": {"value": 0.40},
        }

        self.assertEqual(VideoWidget._margin_box_norm(raw), (0.10, 0.20, 0.70, 0.60))

    def test_margin_box_clamp_preserves_opposing_edge(self):
        left, top, right, bottom = VideoWidget._clamp_margin_values(
            0.90, 0.90, 0.30, 0.30, prefer_left=True, prefer_top=True)

        self.assertAlmostEqual(left, 0.70)
        self.assertAlmostEqual(top, 0.70)
        self.assertAlmostEqual(right, 0.30)
        self.assertAlmostEqual(bottom, 0.30)

    def test_effect_has_margin_box_from_class_name(self):
        self.widget.transforming_effect = None
        self.widget.transforming_effect_object = types.SimpleNamespace(
            info=types.SimpleNamespace(class_name="Blur"))

        self.assertTrue(VideoWidget._effect_has_margin_box(self.widget))

    def test_tracked_object_resolver_uses_evaluated_selected_index(self):
        self.widget.transforming_effect = types.SimpleNamespace(
            data={
                "selected_object_index": {
                    "Points": [{"co": {"X": 1, "Y": 0}}]
                }
            }
        )
        objects = {
            "effect-uuid-0": {"visible": {"value": 1}, "name": "zero"},
            "effect-uuid-2": {"visible": {"value": 1}, "name": "two"},
        }
        raw = {"selected_object_index": {"value": 2}}

        object_id, props = VideoWidget._resolve_tracked_object(self.widget, objects, raw)

        self.assertEqual(object_id, "effect-uuid-2")
        self.assertEqual(props["name"], "two")

    def test_tracked_object_resolver_normalizes_float_selected_index(self):
        self.widget.transforming_effect = None
        objects = {
            "effect-uuid-1": {"visible": {"value": 1}, "name": "one"},
        }
        raw = {"selected_object_index": {"value": 1.0}}

        object_id, props = VideoWidget._resolve_tracked_object(self.widget, objects, raw)

        self.assertEqual(object_id, "effect-uuid-1")
        self.assertEqual(props["name"], "one")

    def test_tracked_object_resolver_ignores_all_selection_for_preview_transform(self):
        self.widget.transforming_effect = None
        objects = {
            "all": {"visible": {"value": 1}, "name": "all objects"},
            "effect-uuid-3": {"visible": {"value": 1}, "name": "three"},
        }
        raw = {"selected_object_index": {"value": -1}}

        object_id, props = VideoWidget._resolve_tracked_object(self.widget, objects, raw)

        self.assertIsNone(object_id)
        self.assertIsNone(props)

    def test_tracked_object_resolver_fallback_skips_all_object(self):
        self.widget.transforming_effect = None
        objects = {
            "all": {"visible": {"value": 1}, "name": "all objects"},
            "effect-uuid-3": {"visible": {"value": 1}, "name": "three"},
        }

        object_id, props = VideoWidget._resolve_tracked_object(self.widget, objects, {})

        self.assertEqual(object_id, "effect-uuid-3")
        self.assertEqual(props["name"], "three")

    def test_update_effect_property_writes_only_changed_tracked_object_property(self):
        class FakeEffect:
            def __init__(self):
                self.data = {
                    "objects": {
                        "effect-uuid-23": {
                            "BaseFPS": {"den": 1, "num": 1},
                            "TimeScale": 1.0,
                            "box_id": "effect-uuid-23",
                            "delta_x": {"value": 0.0},
                            "delta_y": {"value": 0.0},
                            "visible": {"value": 1},
                            "x1": {"value": 0.25},
                        }
                    }
                }
                self.saved = False

            def save(self):
                self.saved = True

        effect = FakeEffect()
        self.widget.transaction_id = None

        with patch("windows.video_widget.Effect.get", return_value=effect):
            VideoWidget.updateEffectProperty(
                self.widget,
                "effect-uuid",
                5,
                "effect-uuid-23",
                "delta_x",
                0.25,
                refresh=False,
            )

        self.assertTrue(effect.saved)
        object_payload = effect.data["objects"]["effect-uuid-23"]
        self.assertEqual(["delta_x"], list(object_payload.keys()))
        points = object_payload["delta_x"]["Points"]
        self.assertEqual(points[0]["co"], {"X": 5, "Y": 0.25})

    def test_update_effect_properties_batches_tracked_object_properties(self):
        class FakeEffect:
            def __init__(self):
                self.data = {
                    "objects": {
                        "effect-uuid-23": {
                            "delta_x": {"Points": []},
                            "delta_y": {"Points": []},
                        }
                    }
                }
                self.save_count = 0

            def save(self):
                self.save_count += 1

        effect = FakeEffect()
        self.widget.transaction_id = None
        refresh_signal = FakeSignal()
        fake_app = types.SimpleNamespace(
            updates=types.SimpleNamespace(transaction_id=None),
            window=types.SimpleNamespace(refreshFrameSignal=refresh_signal),
        )

        with patch("windows.video_widget.Effect.get", return_value=effect), \
             patch("windows.video_widget.get_app", return_value=fake_app):
            VideoWidget.updateEffectProperties(
                self.widget,
                "effect-uuid",
                5,
                "effect-uuid-23",
                {"delta_x": 0.25, "delta_y": -0.5},
            )

        self.assertEqual(effect.save_count, 1)
        object_payload = effect.data["objects"]["effect-uuid-23"]
        self.assertEqual(["delta_x", "delta_y"], list(object_payload.keys()))
        self.assertEqual(object_payload["delta_x"]["Points"][0]["co"], {"X": 5, "Y": 0.25})
        self.assertEqual(object_payload["delta_y"]["Points"][0]["co"], {"X": 5, "Y": -0.5})
        self.assertEqual(refresh_signal.count, 1)

    def test_update_effect_property_refreshes_preview_for_tracked_object(self):
        class FakeEffect:
            def __init__(self):
                self.data = {"objects": {"effect-uuid-23": {"delta_x": {"Points": []}}}}
                self.saved = False

            def save(self):
                self.saved = True

        effect = FakeEffect()
        refresh_signal = FakeSignal()
        fake_app = types.SimpleNamespace(
            updates=types.SimpleNamespace(transaction_id=None),
            window=types.SimpleNamespace(refreshFrameSignal=refresh_signal),
        )
        self.widget.transaction_id = None

        with patch("windows.video_widget.Effect.get", return_value=effect), \
             patch("windows.video_widget.get_app", return_value=fake_app):
            VideoWidget.updateEffectProperty(
                self.widget,
                "effect-uuid",
                5,
                "effect-uuid-23",
                "delta_x",
                0.25,
            )

        self.assertTrue(effect.saved)
        self.assertEqual(refresh_signal.count, 1)

    def test_tracked_object_color_update_initializes_sparse_object_color(self):
        class FakeEffect:
            def __init__(self):
                self.data = {
                    "objects": {
                        "effect-uuid-23": {
                            "delta_x": {"Points": []},
                        }
                    }
                }
                self.saved = False

            def save(self):
                self.saved = True

        effect = FakeEffect()
        model = ClipStandardItemModel()
        label_item = QStandardItem("Border")
        value_item = QStandardItem("")
        label_item.setData((
            "stroke",
            {
                "type": "color",
                "closest_point_x": 1,
                "previous_point_x": 1,
                "object_id": "effect-uuid-23",
                "red": {"value": 62, "Points": []},
                "green": {"value": 143, "Points": []},
                "blue": {"value": 0, "Points": []},
            }
        ))
        value_item.setData([("effect-uuid", "effect")])
        model.appendRow([label_item, value_item])

        helper = PropertiesModel.__new__(PropertiesModel)
        helper.model = model
        helper.frame_number = 5
        helper.parent = FakePropertiesParent(model)
        helper._trim_preview_mode = False
        helper.ignore_update_signal = False
        fake_app = types.SimpleNamespace(
            window=types.SimpleNamespace(refreshFrameSignal=FakeSignal()),
            _tr=lambda text: text,
        )

        with patch("windows.models.properties_model.Effect.get", return_value=effect), \
             patch("windows.models.properties_model.get_app", return_value=fake_app):
            PropertiesModel.color_update(helper, value_item, QColor(10, 20, 30, 255))

        self.assertTrue(effect.saved)
        self.assertEqual(["objects"], list(effect.data.keys()))
        object_payload = effect.data["objects"]["effect-uuid-23"]
        self.assertEqual(["stroke"], list(object_payload.keys()))
        self.assertEqual(
            object_payload["stroke"]["red"]["Points"][0]["co"],
            {"X": 5, "Y": 10},
        )
        self.assertEqual(
            object_payload["stroke"]["green"]["Points"][0]["co"],
            {"X": 5, "Y": 20},
        )
        self.assertEqual(
            object_payload["stroke"]["blue"]["Points"][0]["co"],
            {"X": 5, "Y": 30},
        )

    def test_tracked_object_property_editor_update_writes_narrow_object_payload(self):
        class FakeEffect:
            def __init__(self):
                self.data = {
                    "objects": {
                        "effect-uuid-23": {
                            "delta_x": {"Points": [{"co": {"X": 5, "Y": 0.0}, "interpolation": 1}]},
                            "delta_y": {"Points": [{"co": {"X": 5, "Y": 0.0}, "interpolation": 1}]},
                        },
                        "effect-uuid-24": {
                            "delta_x": {"Points": [{"co": {"X": 5, "Y": 0.5}, "interpolation": 1}]},
                        },
                    }
                }
                self.saved = False

            def save(self):
                self.saved = True

        effect = FakeEffect()
        model = ClipStandardItemModel()
        label_item = QStandardItem("Displacement X-axis")
        value_item = QStandardItem("")
        label_item.setData((
            "delta_x",
            {
                "type": "float",
                "closest_point_x": 5,
                "previous_point_x": 5,
                "object_id": "effect-uuid-23",
                "choices": [],
            }
        ))
        value_item.setData([("effect-uuid", "effect")])
        model.appendRow([label_item, value_item])

        helper = PropertiesModel.__new__(PropertiesModel)
        helper.model = model
        helper.frame_number = 5
        helper.parent = FakePropertiesParent(model)
        helper._trim_preview_mode = False
        helper.ignore_update_signal = False
        fake_app = types.SimpleNamespace(
            window=types.SimpleNamespace(refreshFrameSignal=FakeSignal()),
            _tr=lambda text: text,
        )

        with patch("windows.models.properties_model.Effect.get", return_value=effect), \
             patch("windows.models.properties_model.get_app", return_value=fake_app):
            PropertiesModel.value_updated(helper, value_item, value=0.25)

        self.assertTrue(effect.saved)
        self.assertEqual(["objects"], list(effect.data.keys()))
        self.assertEqual(["effect-uuid-23"], list(effect.data["objects"].keys()))
        object_payload = effect.data["objects"]["effect-uuid-23"]
        self.assertEqual(["delta_x"], list(object_payload.keys()))
        self.assertEqual(
            object_payload["delta_x"]["Points"][0]["co"],
            {"X": 5, "Y": 0.25},
        )

    def test_all_tracked_objects_property_update_writes_all_payload(self):
        class FakeEffect:
            def __init__(self):
                self.data = {
                    "objects": {
                        "effect-uuid-23": {
                            "background_alpha": {"Points": [{"co": {"X": 1, "Y": 0.15}, "interpolation": 1}]},
                            "delta_x": {"Points": [{"co": {"X": 1, "Y": 0.0}, "interpolation": 1}]},
                            "stroke": {
                                "red": {"Points": [{"co": {"X": 1, "Y": 10}, "interpolation": 1}]},
                                "green": {"Points": [{"co": {"X": 1, "Y": 20}, "interpolation": 1}]},
                                "blue": {"Points": [{"co": {"X": 1, "Y": 30}, "interpolation": 1}]},
                            },
                        },
                        "effect-uuid-24": {
                            "background_alpha": {"Points": [{"co": {"X": 1, "Y": 0.15}, "interpolation": 1}]},
                        },
                    }
                }
                self.saved = False

            def save(self):
                self.saved = True

        effect = FakeEffect()
        model = ClipStandardItemModel()
        label_item = QStandardItem("Background Alpha")
        value_item = QStandardItem("")
        label_item.setData((
            "background_alpha",
            {
                "type": "float",
                "closest_point_x": 1,
                "previous_point_x": 1,
                "object_id": "all",
                "choices": [],
            }
        ))
        value_item.setData([("effect-uuid", "effect")])
        model.appendRow([label_item, value_item])

        helper = PropertiesModel.__new__(PropertiesModel)
        helper.model = model
        helper.frame_number = 1
        helper.parent = FakePropertiesParent(model)
        helper._trim_preview_mode = False
        helper.ignore_update_signal = False
        fake_app = types.SimpleNamespace(
            window=types.SimpleNamespace(refreshFrameSignal=FakeSignal()),
            _tr=lambda text: text,
            project=types.SimpleNamespace(get=lambda key: {"num": 30, "den": 1}),
        )

        with patch("windows.models.properties_model.Effect.get", return_value=effect), \
             patch("windows.models.properties_model.get_app", return_value=fake_app):
            PropertiesModel.value_updated(helper, value_item, value=0.35)

        self.assertTrue(effect.saved)
        self.assertEqual(["objects"], list(effect.data.keys()))
        self.assertIn("all", effect.data["objects"])
        self.assertIn("effect-uuid-23", effect.data["objects"])
        self.assertIn("effect-uuid-24", effect.data["objects"])
        object_payload = effect.data["objects"]["all"]
        self.assertEqual(["background_alpha"], list(object_payload.keys()))
        self.assertEqual(
            object_payload["background_alpha"]["Points"][0]["co"],
            {"X": 1, "Y": 0.35},
        )
        self.assertEqual(
            effect.data["objects"]["effect-uuid-23"]["background_alpha"]["Points"][0]["co"],
            {"X": 1, "Y": 0.35},
        )

    def test_all_tracked_objects_transform_update_writes_all_payload(self):
        class FakeEffect:
            def __init__(self):
                self.data = {
                    "objects": {
                        "effect-uuid-23": {
                            "delta_x": {"Points": [{"co": {"X": 5, "Y": 0.0}, "interpolation": 1}]},
                        },
                        "effect-uuid-24": {
                            "delta_x": {"Points": [{"co": {"X": 5, "Y": 0.25}, "interpolation": 1}]},
                        },
                    }
                }
                self.saved = False

            def save(self):
                self.saved = True

        effect = FakeEffect()
        model = ClipStandardItemModel()
        label_item = QStandardItem("Displacement X-axis")
        value_item = QStandardItem("")
        label_item.setData((
            "delta_x",
            {
                "type": "float",
                "closest_point_x": 5,
                "previous_point_x": 5,
                "object_id": "all",
                "choices": [],
            }
        ))
        value_item.setData([("effect-uuid", "effect")])
        model.appendRow([label_item, value_item])

        helper = PropertiesModel.__new__(PropertiesModel)
        helper.model = model
        helper.frame_number = 5
        helper.parent = FakePropertiesParent(model)
        helper._trim_preview_mode = False
        helper.ignore_update_signal = False
        fake_app = types.SimpleNamespace(
            window=types.SimpleNamespace(refreshFrameSignal=FakeSignal()),
            _tr=lambda text: text,
            project=types.SimpleNamespace(get=lambda key: {"num": 30, "den": 1}),
        )

        with patch("windows.models.properties_model.Effect.get", return_value=effect), \
             patch("windows.models.properties_model.get_app", return_value=fake_app):
            PropertiesModel.value_updated(helper, value_item, value=0.5)

        self.assertTrue(effect.saved)
        self.assertEqual(["objects"], list(effect.data.keys()))
        self.assertIn("all", effect.data["objects"])
        self.assertIn("effect-uuid-23", effect.data["objects"])
        self.assertIn("effect-uuid-24", effect.data["objects"])
        object_payload = effect.data["objects"]["all"]
        self.assertEqual(["delta_x"], list(object_payload.keys()))
        self.assertEqual(
            object_payload["delta_x"]["Points"][0]["co"],
            {"X": 5, "Y": 0.5},
        )
        self.assertEqual(
            effect.data["objects"]["effect-uuid-24"]["delta_x"]["Points"][0]["co"],
            {"X": 5, "Y": 0.5},
        )

    def test_all_tracked_objects_color_update_writes_all_payload(self):
        class FakeEffect:
            def __init__(self):
                self.data = {
                    "objects": {
                        "effect-uuid-23": {
                            "stroke": {
                                "red": {"Points": [{"co": {"X": 1, "Y": 10}, "interpolation": 1}]},
                                "green": {"Points": [{"co": {"X": 1, "Y": 20}, "interpolation": 1}]},
                                "blue": {"Points": [{"co": {"X": 1, "Y": 30}, "interpolation": 1}]},
                            },
                        }
                    }
                }
                self.saved = False

            def save(self):
                self.saved = True

        effect = FakeEffect()
        model = ClipStandardItemModel()
        label_item = QStandardItem("Border")
        value_item = QStandardItem("")
        label_item.setData((
            "stroke",
            {
                "type": "color",
                "closest_point_x": 1,
                "previous_point_x": 1,
                "object_id": "all",
                "red": {"value": 10, "Points": []},
                "green": {"value": 20, "Points": []},
                "blue": {"value": 30, "Points": []},
            }
        ))
        value_item.setData([("effect-uuid", "effect")])
        model.appendRow([label_item, value_item])

        helper = PropertiesModel.__new__(PropertiesModel)
        helper.model = model
        helper.frame_number = 1
        helper.parent = FakePropertiesParent(model)
        helper._trim_preview_mode = False
        helper.ignore_update_signal = False
        fake_app = types.SimpleNamespace(
            window=types.SimpleNamespace(refreshFrameSignal=FakeSignal()),
            _tr=lambda text: text,
        )

        with patch("windows.models.properties_model.Effect.get", return_value=effect), \
             patch("windows.models.properties_model.get_app", return_value=fake_app):
            PropertiesModel.color_update(helper, value_item, QColor(100, 110, 120, 255))

        self.assertTrue(effect.saved)
        self.assertEqual(["objects"], list(effect.data.keys()))
        self.assertIn("all", effect.data["objects"])
        self.assertIn("effect-uuid-23", effect.data["objects"])
        object_payload = effect.data["objects"]["all"]
        self.assertEqual(["stroke"], list(object_payload.keys()))
        self.assertEqual(object_payload["stroke"]["red"]["Points"][0]["co"], {"X": 1, "Y": 100})
        self.assertEqual(object_payload["stroke"]["green"]["Points"][0]["co"], {"X": 1, "Y": 110})
        self.assertEqual(object_payload["stroke"]["blue"]["Points"][0]["co"], {"X": 1, "Y": 120})
        self.assertEqual(effect.data["objects"]["effect-uuid-23"]["stroke"]["red"]["Points"][0]["co"], {"X": 1, "Y": 100})

    def test_all_tracked_objects_update_saves_only_edited_property(self):
        class FakeEffect:
            def __init__(self):
                self.data = {
                    "objects": {
                        "all": {
                            "stroke_width": {"Points": [{"co": {"X": 1, "Y": 8.0}, "interpolation": 1}]},
                            "stroke_alpha": {"Points": [{"co": {"X": 1, "Y": 0.7}, "interpolation": 1}]},
                        },
                    }
                }
                self.saved = False

            def save(self):
                self.saved = True

        effect = FakeEffect()
        model = ClipStandardItemModel()
        label_item = QStandardItem("Stroke alpha")
        value_item = QStandardItem("")
        label_item.setData((
            "stroke_alpha",
            {
                "type": "float",
                "closest_point_x": 1,
                "previous_point_x": 1,
                "object_id": "all",
                "choices": [],
            }
        ))
        value_item.setData([("effect-uuid", "effect")])
        model.appendRow([label_item, value_item])

        helper = PropertiesModel.__new__(PropertiesModel)
        helper.model = model
        helper.frame_number = 1
        helper.parent = FakePropertiesParent(model)
        helper._trim_preview_mode = False
        helper.ignore_update_signal = False
        fake_app = types.SimpleNamespace(
            window=types.SimpleNamespace(refreshFrameSignal=FakeSignal()),
            _tr=lambda text: text,
            project=types.SimpleNamespace(get=lambda key: {"num": 30, "den": 1}),
        )

        with patch("windows.models.properties_model.Effect.get", return_value=effect), \
             patch("windows.models.properties_model.get_app", return_value=fake_app):
            PropertiesModel.value_updated(helper, value_item, value=0.25)

        object_payload = effect.data["objects"]["all"]
        self.assertEqual(["stroke_alpha"], list(object_payload.keys()))
        self.assertEqual(object_payload["stroke_alpha"]["Points"][0]["co"], {"X": 1, "Y": 0.25})

    def test_all_tracked_objects_update_seeds_missing_keyframe_property(self):
        class FakeEffect:
            def __init__(self):
                self.data = {
                    "objects": {
                        "all": {
                            "stroke_width": {"Points": [{"co": {"X": 1, "Y": 8.0}, "interpolation": 1}]},
                        },
                        "effect-uuid-23": {
                            "stroke_alpha": {"Points": [{"co": {"X": 1, "Y": 0.7}, "interpolation": 1}]},
                        },
                    }
                }
                self.saved = False

            def save(self):
                self.saved = True

        effect = FakeEffect()
        model = ClipStandardItemModel()
        label_item = QStandardItem("Stroke alpha")
        value_item = QStandardItem("")
        label_item.setData((
            "stroke_alpha",
            {
                "type": "float",
                "closest_point_x": 1,
                "previous_point_x": 1,
                "object_id": "all",
                "choices": [],
                "Points": [{"co": {"X": 1, "Y": 0.7}, "interpolation": 1}],
            }
        ))
        value_item.setData([("effect-uuid", "effect")])
        model.appendRow([label_item, value_item])

        helper = PropertiesModel.__new__(PropertiesModel)
        helper.model = model
        helper.frame_number = 1
        helper.parent = FakePropertiesParent(model)
        helper._trim_preview_mode = False
        helper.ignore_update_signal = False
        fake_app = types.SimpleNamespace(
            window=types.SimpleNamespace(refreshFrameSignal=FakeSignal()),
            _tr=lambda text: text,
            project=types.SimpleNamespace(get=lambda key: {"num": 30, "den": 1}),
        )

        with patch("windows.models.properties_model.Effect.get", return_value=effect), \
             patch("windows.models.properties_model.get_app", return_value=fake_app):
            PropertiesModel.value_updated(helper, value_item, value=0.25)

        object_payload = effect.data["objects"]["all"]
        self.assertEqual(["stroke_alpha"], list(object_payload.keys()))
        self.assertIsInstance(object_payload["stroke_alpha"], dict)
        self.assertEqual(object_payload["stroke_alpha"]["Points"][0]["co"], {"X": 1, "Y": 0.25})

    def test_tracked_object_transform_modes_exclude_origin_and_rotation(self):
        QWidget.__init__(self.widget)
        self.widget.transform = QTransform()
        self.widget.mouse_dragging = False
        self.widget.transform_mode = None
        self.widget.resize_button = types.SimpleNamespace(isVisible=lambda: False)
        self.widget.cursors = {}
        self.widget.transforming_effect = types.SimpleNamespace(data={})
        self.widget.transforming_effect_object = types.SimpleNamespace(
            info=types.SimpleNamespace(has_tracked_object=True, class_name="ObjectDetection")
        )
        self.widget.centerHandle = QRectF(44, 44, 12, 12)
        self.widget.topLeftHandle = QRectF(0, 0, 12, 12)
        self.widget.topRightHandle = QRectF(88, 0, 12, 12)
        self.widget.bottomLeftHandle = QRectF(0, 88, 12, 12)
        self.widget.bottomRightHandle = QRectF(88, 88, 12, 12)
        self.widget.topHandle = QRectF(44, 0, 12, 12)
        self.widget.bottomHandle = QRectF(44, 88, 12, 12)
        self.widget.leftHandle = QRectF(0, 44, 12, 12)
        self.widget.rightHandle = QRectF(88, 44, 12, 12)
        self.widget.topShearHandle = QRectF(0, 0, 100, 12)
        self.widget.leftShearHandle = QRectF(0, 0, 12, 100)
        self.widget.rightShearHandle = QRectF(88, 0, 12, 100)
        self.widget.bottomShearHandle = QRectF(0, 88, 100, 12)
        self.widget.clipBounds = QRectF(0, 0, 100, 100)

        VideoWidget.checkTransformMode(self.widget, 0, 0, 0, mouse_event_at(50, 50))
        self.assertEqual(self.widget.hover_transform_mode, "location")

        VideoWidget.checkTransformMode(self.widget, 0, 0, 0, mouse_event_at(150, 150))
        self.assertIsNone(self.widget.hover_transform_mode)
        self.assertEqual(self.widget.hover_cursor, Qt.ArrowCursor)


if __name__ == "__main__":
    unittest.main()



"""
 @file
 @brief This file contains the credits treeview, used by the about window
 @author Noah Figg <eggmunkee@hotmail.com>
 @author Jonathan Thomas <jonathan@openshot.org>

 @section LICENSE

 Copyright (c) 2008-2018 OpenShot Studios, LLC
 (http://www.openshotstudios.com). This file is part of
 OpenShot Video Editor (http://www.openshot.org), an open-source project
 dedicated to delivering high quality video editing and animation solutions
 to the world.

 OpenShot Video Editor is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 OpenShot Video Editor is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
 """

import webbrowser
from qt_api import Qt
from qt_api import QTreeView, QAbstractItemView, QSizePolicy, QHeaderView, QApplication
from qt_api import QCursor
from qt_api import make_filter_regex, set_proxy_filter
from functools import partial

from classes.logger import log
from classes.app import get_app
from windows.models.credits_model import CreditsModel
from .menu import StyledContextMenu


class CreditsTreeView(QTreeView):
    """ A ListView QWidget used on the credits window """
    def resize_contents(self):
        pass

    def refresh_view(self):
        """Format columns and sort"""
        # Model is already populated; just adjust view

        # Format columns
        self.header().setSectionResizeMode(0, QHeaderView.Fixed)
        self.header().setSectionResizeMode(1, QHeaderView.Fixed)
        self.setColumnWidth(0, 22)
        self.setColumnWidth(1, 22)
        self.setColumnWidth(2, 150)
        self.setColumnWidth(3, 150)
        self.setColumnWidth(4, 150)
        self.sortByColumn(2, Qt.AscendingOrder)

        if "email" not in self.columns:
            self.setColumnHidden(3, True)
        if "website" not in self.columns:
            self.setColumnHidden(4, True)

    def filter_changed(self, text=""):
        """Apply filter text to proxy model"""
        regex = make_filter_regex(text, case_insensitive=True)
        set_proxy_filter(self.model(), regex)
        self.model().setFilterKeyColumn(-1)
        self.model().sort(2, Qt.AscendingOrder)

    def contextMenuEvent(self, event):
        log.info('contextMenuEvent')
        _ = get_app()._tr

        # Get proxy model and selection
        model = self.model()
        index = self.indexAt(event.pos())
        if index.isValid():
            email = model.index(index.row(), 3).data()
            website = model.index(index.row(), 4).data()

            menu = StyledContextMenu(parent=self)
            if email:
                copy_action = menu.addAction(_("Copy E-mail"))
                copy_action.triggered.connect(partial(self.CopyEmailTriggered, email))
            if website:
                github_action = menu.addAction(_("View Website"))
                github_action.triggered.connect(partial(self.ViewWebsite, website))
            menu.popup(QCursor.pos())

    def CopyEmailTriggered(self, email=""):
        log.info("CopyEmailTriggered")
        clipboard = QApplication.clipboard()
        clipboard.setText(email)

    def ViewWebsite(self, website=""):
        log.info("ViewWebsite")
        try:
            webbrowser.open(website)
        except Exception:
            log.warning('Failed to launch web browser to %s' % website)

    def __init__(self, credits, columns, *args):
        # Invoke parent init
        QTreeView.__init__(self, *args)

        # Get a reference to the window object
        self.win = get_app().window

        # Get Model data
        self.credits_model = CreditsModel(credits)
        self.selected = []

        # Setup header columns
        self.setIndentation(0)
        self.setSelectionBehavior(QTreeView.SelectRows)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.setWordWrap(True)
        self.columns = columns

        # Populate model
        self.credits_model.update_model()

        # Set proxy model and shared selection model
        self.setModel(self.credits_model.proxy_model)
        self.selectionModel().deleteLater()
        self.setSelectionModel(self.credits_model.selection_model)

        # Refresh view
        self.refresh_view()



"""Tests for shared QMessageBox presentation."""

import unittest

from qt_api import QApplication, QMessageBox, QObject

from themes.base import BaseTheme, MessageBoxStyleFilter


app = QApplication.instance() or QApplication([])


class MessageBoxStylingTests(unittest.TestCase):
    def tearDown(self):
        for widget in QApplication.topLevelWidgets():
            if isinstance(widget, QMessageBox):
                widget.close()

    def test_assigns_semantic_button_roles_and_removes_icons(self):
        box = QMessageBox(
            QMessageBox.Question,
            "Unsaved Changes",
            "Save changes to project before closing?",
            QMessageBox.Cancel | QMessageBox.No | QMessageBox.Yes,
        )

        MessageBoxStyleFilter.style_message_box(box)

        self.assertEqual(
            box.button(QMessageBox.Yes).property("dialogRole"), "primary")
        self.assertEqual(
            box.button(QMessageBox.No).property("dialogRole"), "secondary")
        self.assertEqual(
            box.button(QMessageBox.Cancel).property("dialogRole"), "cancel")
        self.assertTrue(box.button(QMessageBox.Yes).icon().isNull())
        self.assertTrue(box.button(QMessageBox.No).icon().isNull())
        self.assertTrue(box.button(QMessageBox.Cancel).icon().isNull())
        self.assertNotIn("&", box.button(QMessageBox.Yes).text())
        self.assertNotIn("&", box.button(QMessageBox.No).text())

        icon_label = box.findChild(QObject, "qt_msgboxex_icon_label")
        self.assertIsNotNone(icon_label)
        self.assertTrue(icon_label.isHidden())
        self.assertEqual(box.minimumWidth(), 400)

    def test_application_filter_styles_future_message_boxes(self):
        theme = BaseTheme(app)
        theme.install_message_box_styling()
        box = QMessageBox(
            QMessageBox.Question,
            "Confirm",
            "Continue?",
            QMessageBox.No | QMessageBox.Yes,
        )

        box.show()
        app.processEvents()

        self.assertEqual(
            box.button(QMessageBox.Yes).property("dialogRole"), "primary")
        self.assertEqual(
            box.button(QMessageBox.No).property("dialogRole"), "secondary")

if __name__ == "__main__":
    unittest.main()



"""
Timeline theme classes for the Humanity Dark and Retro Qt themes.

Subclass any theme and override only what you need:

    class MyTheme(HumanityDarkTimelineTheme):
        def __init__(self):
            super().__init__()
            self.clip.background = QColor("#ff0000")
            self.track.height    = 80
"""

from qt_api import QColor

from windows.views.timeline_backend.theme import TimelineTheme, _icon


class HumanityDarkTimelineTheme(TimelineTheme):
    """Humanity Dark timeline theme — base for all other themes."""

    def __init__(self):
        super().__init__()

        # ── Timeline ──────────────────────────────────────────────────────
        self.background             = QColor("#191919")
        self.background2            = QColor()
        self.playhead_color         = QColor("#FF0024")
        self.playhead_width         = 2.0
        self.clip_selected          = QColor("#FF0000")
        self.selection              = QColor(42, 130, 218, 102)
        self.selection_border       = QColor(42, 130, 218, 102)
        self.selection_border_width = 1.0
        self.playback_cache_color   = QColor("#4B92AD")
        self.playback_cache_height  = 5.0
        self.ruler_name_background  = QColor("#191919")
        self.ruler_name_background2 = QColor()
        self.ruler_time_font_size   = 13
        self.ruler_time_pad_left    = 17
        self.ruler_time_pad_top     = 12
        self.ruler_label_top        = 6
        self.scrollbar_handle       = QColor("#4B92AD")
        self.scrollbar_track        = QColor("#000000")
        self.scrollbar_width        = 6
        self.waveform_color         = QColor("#2A82DA")
        self.waveform_peak_color    = QColor(42, 130, 218, 128)
        self.keyframe_fill          = QColor("#4D7BFF")
        self.keyframe_border        = QColor("#FFFFFF")
        self.keyframe_inactive_opacity       = 0.7
        self.keyframe_size                   = 10
        self.keyframe_panel_property_bg      = QColor("#2F2F2F")
        self.keyframe_panel_row_border_color = QColor(0, 0, 0, 0)
        self.keyframe_panel_row_border_width = 0.0
        self.keyframe_panel_curve_color      = QColor("#4B92AD")
        self.keyframe_panel_marker_fill      = QColor("#4B92AD")
        self.keyframe_panel_marker_border    = QColor("#7DC3DD")

        # ── Clip ──────────────────────────────────────────────────────────
        self.clip.background    = QColor("#525252")
        self.clip.background2   = QColor("#222628")
        self.clip.top_overlay   = QColor(255, 255, 255, 51)
        self.clip.top_overlay2  = QColor(255, 255, 255, 0)
        self.clip.border_color  = QColor("#4B92AD")
        self.clip.border_radius = 8
        self.clip.border_width  = 2.0
        self.clip.font_color    = QColor("#FFFFFF")
        self.clip.font_size     = 9
        self.clip.height        = 64
        self.clip.shadow_color  = QColor("#000000")
        self.clip.shadow_blur   = 10

        # ── Track ─────────────────────────────────────────────────────────
        self.track.background               = QColor("#060606")
        self.track.background2              = QColor("#323232")
        self.track.border_color             = QColor("#4B92AD")
        self.track.border_radius            = 0
        self.track.height                   = 62
        self.track.gap                      = 8
        self.track.margin_top               = -1
        self.track.font_color               = QColor("#FFFFFF")
        self.track.font_size                = 9
        self.track.name_background          = QColor("#000000")
        self.track.name_width               = 140
        self.track.name_border_color        = QColor("#4B92AD")
        self.track.name_border_width        = 1
        self.track.name_border_top_color    = QColor("#4B92AD")
        self.track.name_border_top_width    = 1
        self.track.name_border_bottom_color = QColor("#4B92AD")
        self.track.name_border_bottom_width = 1
        self.track.name_radius_tl           = 8
        self.track.name_radius_bl           = 8
        self.track.name_top_overlay         = QColor(255, 255, 255, 51)
        self.track.name_top_overlay2        = QColor(255, 255, 255, 0)

        # ── Transition ────────────────────────────────────────────────────
        self.transition.background       = QColor("#0192C1")
        self.transition.background2      = QColor("#3FA1BF")
        self.transition.border_color     = QColor("#0192C1")
        self.transition.border_radius    = 8
        self.transition.border_width     = 2.0
        self.transition.font_color       = QColor("#FFFFFF")
        self.transition.font_size        = 9
        self.transition.height           = 64
        self.transition.background_image = _icon("themes/humanity/images/transition.svg")

        # ── Ruler ─────────────────────────────────────────────────────────
        self.ruler.background   = QColor("#191919")
        self.ruler.background2  = QColor()
        self.ruler.border_color = QColor("#ACACAC")
        self.ruler.font_color   = QColor("#999999")
        self.ruler.font_size    = 10
        self.ruler.height       = 39

        # ── Icons ─────────────────────────────────────────────────────────
        _h = "themes/humanity/images/"
        _c = "themes/cosmic/images/"    # fallback for icons missing a Humanity Dark variant

        self.menu_size               = 12
        self.menu_margin             = 4
        self.playhead_icon           = _icon("themes/humanity/images/playhead.svg")
        self.playhead_icon_width     = 12
        self.playhead_icon_height    = 188
        self.playhead_icon_offset_x  = -6
        self.playhead_icon_offset_y  = 20
        self.marker_icon             = _icon("themes/humanity/images/marker.svg")
        self.marker_icon_width       = 8
        self.marker_icon_height      = 10
        self.marker_icon_offset_x    = -4
        self.marker_icon_offset_y    = 0

        self.track_keyframe_panel_disabled_icon = _icon(_h + "humanity-dark-track-keyframe-panel-show-disabled.svg")
        self.track_keyframe_panel_enabled_icon  = _icon(_h + "humanity-dark-track-keyframe-panel-show-enabled.svg")
        self.keyframe_panel_add_icon            = _icon(_c + "keyframe-panel-add.svg")  # no Humanity Dark variant
        self.track_add_above_disabled_icon      = _icon(_h + "track-add-above-disabled.svg")
        self.track_add_above_enabled_icon       = _icon(_h + "track-add-above-enabled.svg")
        self.track_add_below_disabled_icon      = _icon(_h + "track-add-below-disabled.svg")
        self.track_add_below_enabled_icon       = _icon(_h + "track-add-below-enabled.svg")
        self.track_delete_disabled_icon         = _icon(_h + "track-delete-disabled.svg")
        self.track_delete_enabled_icon          = _icon(_h + "track-delete-enabled.svg")
        self.track_locked_disabled_icon         = _icon(_h + "humanity-dark-track-locked-disabled.svg")
        self.track_locked_enabled_icon          = _icon(_h + "humanity-dark-track-locked-enabled.svg")
        self.track_unlocked_disabled_icon       = _icon(_h + "humanity-dark-track-unlocked-disabled.svg")
        self.track_unlocked_enabled_icon        = _icon(_h + "humanity-dark-track-unlocked-enabled.svg")

        self.keyframe_toggle_off_icon = self.track_keyframe_panel_disabled_icon
        self.keyframe_toggle_on_icon  = self.track_keyframe_panel_enabled_icon


class RetroTimelineTheme(HumanityDarkTimelineTheme):
    """Retro (light) timeline theme."""

    def __init__(self):
        super().__init__()

        # ── Timeline ──────────────────────────────────────────────────────
        self.background             = QColor("#F0F0F0")
        self.background2            = QColor()
        self.ruler_name_background  = QColor("#0A070A")
        self.ruler_name_background2 = QColor("#3C3C3C")
        self.keyframe_inactive_opacity    = 0.72
        self.keyframe_panel_property_bg   = QColor("#E5E7EA")
        self.keyframe_panel_marker_border = QColor("#3A748A")

        # ── Clip ──────────────────────────────────────────────────────────
        self.clip.background    = QColor("#FEDC66")
        self.clip.background2   = QColor()
        self.clip.top_overlay   = QColor()      # gradient overlay disabled
        self.clip.top_overlay2  = QColor()
        self.clip.border_color  = QColor("#CD8D00")
        self.clip.border_radius = 0
        self.clip.font_color    = QColor("#FFFFFF")

        # ── Track ─────────────────────────────────────────────────────────
        self.track.background        = QColor("#E5E7EA")
        self.track.background2       = QColor()
        self.track.border_radius     = 0
        self.track.font_color        = QColor("#000000")
        self.track.name_background   = QColor("#DEDDDD")
        self.track.name_radius_tl    = 0         # gradient overlay/radius disabled
        self.track.name_radius_bl    = 0
        self.track.name_top_overlay  = QColor()
        self.track.name_top_overlay2 = QColor()

        # ── Transition ────────────────────────────────────────────────────
        self.transition.border_radius = 0

        # ── Ruler ─────────────────────────────────────────────────────────
        self.ruler.background  = QColor("#0A070A")
        self.ruler.background2 = QColor("#3C3C3C")
        self.ruler.font_color  = QColor("#C9C9C9")

        # ── Icons ─────────────────────────────────────────────────────────
        _h = "themes/humanity/images/"

        self.track_keyframe_panel_disabled_icon = _icon(_h + "retro-track-keyframe-panel-show-disabled.svg")
        self.track_keyframe_panel_enabled_icon  = _icon(_h + "retro-track-keyframe-panel-show-enabled.svg")
        self.keyframe_panel_add_icon            = _icon(_h + "keyframe-panel-add.svg")
        self.track_locked_disabled_icon         = _icon(_h + "retro-track-locked-disabled.svg")
        self.track_locked_enabled_icon          = _icon(_h + "retro-track-locked-enabled.svg")
        self.track_unlocked_disabled_icon       = _icon(_h + "retro-track-unlocked-disabled.svg")
        self.track_unlocked_enabled_icon        = _icon(_h + "retro-track-unlocked-enabled.svg")

        self.keyframe_toggle_off_icon = self.track_keyframe_panel_disabled_icon
        self.keyframe_toggle_on_icon  = self.track_keyframe_panel_enabled_icon
